from __future__ import print_function

import click
import fileinput
import os
import subprocess
import sys
import time

from obsctl.lib import obsc
from obsctl.lib import spec
from obsctl.lib import tar
from obsctl.cli import pass_context


@click.command('scratchbuild',
               help='Executes a local scratch build for OBS projects '
               'given a tarball',
               short_help='Executes local OBS scratch build')
@click.option('--source-file', '-f', help='Path to source tarball file', required=True)
@click.option('--continuous-merge', '-m', help='Enable continuous merge mode',
              is_flag=True)
@click.option('--repo-type', '-T', help='Source repository type',
              type=click.Choice(['git', 'hg', 'svn']), default='git')
@click.option('--repo-owner', '-o', help='Source repository owner',
              required=True)
@click.option('--repo-name', '-n', help='Source repository name')
@click.option('--repo-reltag', '-t', help='Source repository release tag',
              required=True)
@click.option('--repo-revision', '-R',
              help='Source repository revision/commit', required=True)
@click.option('--repo-branch', '-B', help='Source repository branch',
              required=True)
@click.option('--revision-form', '-r', help='Form of version for spec',
              type=click.Choice(['prerel', 'postrel']), default='postrel')
@click.option('--obs-project', '-s', help='Open Build Service project space',
              required=True)
@click.option('--obs-package', '-p', help='Open Build Service project package',
              required=True)
@click.option('--create-obs-pkg-if-none', '-c',
              help='Create package space in OBS if it doesn\'t exist',
              is_flag=True)
@click.option('--build-distribution', '-d', required=True,
              help='Target distribution to build for')
@click.option('--insecure', '-i',
              help='Ignore issues with HTTPS (self signed certs, etc.)',
              is_flag=True)
@pass_context
def cli(ctx, source_file, continuous_merge, repo_type, repo_owner, repo_name,
        repo_reltag, repo_revision, repo_branch, revision_form, obs_project,
        obs_package, create_obs_pkg_if_none, build_distribution, insecure):
    # TODO: Simplify this, there must be some way to capture all arguments in
    #       a map or split things out more logically, this is for the sake of
    #       time.
    args = {
            'src_tarball': source_file,
            'ci_mode': continuous_merge,
            'obs_server_addr': ctx.obs_server,
            'obs_auth_user': ctx.obs_user,
            'obs_auth_pass': ctx.obs_pass,
            'src_repo_type': repo_type,
            'src_repo_owner': repo_owner,
            'src_repo_name': repo_name,
            'src_repo_tag': repo_reltag,
            'src_repo_rev': repo_revision,
            'src_repo_branch': repo_branch,
            'spec_rev_form': revision_form,
            'obs_project': obs_project,
            'obs_package': obs_package,
            'create_obs_pkg_if_none': create_obs_pkg_if_none,
            'build_dist': build_distribution,
            'insecure': insecure,
            'commit_time': time.strftime('%Y%m%d%H%M%S')
            }
    main(args)


def execute(cmd, dir):
    curr_dir = os.getcwd()
    os.chdir(dir)
    popen = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                             universal_newlines=True, shell=True)
    for stdout_line in iter(popen.stdout.readline, ""):
        yield stdout_line
    popen.stdout.close()
    os.chdir(curr_dir)
    return_code = popen.wait()

    if return_code:
        raise subprocess.CalledProcessError(return_code, cmd)


def main(args):
    oscconf = obsc.configure(args['obs_server_addr'],
                             args['obs_auth_user'],
                             args['obs_auth_pass'],
                             insecure_connection=True)

    obspkg = obsc.checkout_obs_package(oscconf,
                                       args['obs_server_addr'],
                                       args['obs_project'],
                                       args['obs_package'],
                                       args['obs_auth_user'],
                                       args['src_repo_owner'],
                                       args['src_repo_name'],
                                       args['create_obs_pkg_if_none'])

    if obspkg is False:
        print("No package space exists, exiting!")
        sys.exit(2)

    spec_snap_version = spec.generate_snapshot_version(args['src_repo_tag'],
                                                       args['src_repo_type'],
                                                       args['src_repo_rev'],
                                                       args['src_repo_branch'],
                                                       args['spec_rev_form'])

    if args['ci_mode'] is True:
        release_tarball_ver = spec_snap_version
    else:
        release_tarball_ver = args['src_repo_tag']

    tar.rebuild_tarball(args['src_tarball'], release_tarball_ver,
                        obspkg.absdir)

    obs_package_service = """
    <services>
      <service name="extract_file">
        <param name="archive">{package}*.tar.gz</param>
        <param name="files">*{package}*/dist/{package}.spec</param>
      </service>
    </services>
    """.format(package=args['obs_package'])

    obs_package_service_file = open(obspkg.absdir + "/_service", 'w')
    obs_package_service_file.write(obs_package_service)
    obs_package_service_file.close()

    # A bit of a workaround but it works :)
    obspkg.filenamelist.append("_service")

    obspkg.run_source_services()

    # Now replace version and release numbers
    service_file = ('{package_dir}/_service:extract_file:{package}.spec'
                    .format(package_dir=obspkg.absdir,
                            package=args['obs_package']))
    try:
        for line in fileinput.input(service_file, inplace=True):
            if "Version:" in line:
                print("Version:        {tag}".format(tag=release_tarball_ver))
            else:
                print(line, end="")
    finally:
        fileinput.close()

    """ Here RIP the attempt to use osc api """
    """
    opts = {
                'no_verify': True,
                'userootforbuild': False,
                'trust_all_projects': True,
                'jobs': '4',
                'root': os.path.join(os.getcwd(),
                                     'scratchbuild-' + args.build_dist)
            }
    build_args = [args.build_dist, 'x86_64',
                  '{}/{}.spec'.format(obs_package.absdir, args.obs_package)]

    osc.build.main(args.obs_server_addr, opts, build_args)
    """
    build_command = ("osc --config={conf} build --jobs=4 --trust-all-projects "
                     "--no-verify --noservice --root={path}/scratchbuild-"
                     "{dist} {dist}".format(conf=oscconf.name,
                                            path=os.getcwd(),
                                            dist=args['build_dist']))
    for path in execute(build_command, obspkg.absdir):
        print(path, end="")

    # Clean up
    if os.path.isfile("{}-{}.tar.gz".format(args['obs_package'],
                                            release_tarball_ver)):
        os.remove("{}-{}.tar.gz".format(args['obs_package'],
                  release_tarball_ver))
    oscconf.close()

    print("Scratchbuild Completed Successfully!!")
